package org.apache.xindice.server.rpc;

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: RPCDefaultMessage.java,v 1.7 2002/12/27 18:37:24 dviner Exp $
 */

import org.apache.xindice.core.Collection;
import org.apache.xindice.core.Database;

/**
 * @author <a href="mailto:kstaken@xmldatabases.org">Kimbro Staken</a>
 */
public abstract class RPCDefaultMessage implements RPCMessage {

   public static final String API_NAME = "Xindice XML-RPC";
   public static final String API_VERSION = "0.1";
   public static final String RESULT = "result";
   public static final String NAME = "name";
   public static final String COLLECTION = "collection";
   public static final String DOCUMENT = "document";
   public static final String COMPRESSED = "compressed";
   public static final String TIMESTAMP = "timestamp";
   public static final String PATTERN = "pattern";
   public static final String MAXKEYSIZE = "maxkeysize";
   public static final String PAGESIZE = "pagesize";
   public static final String TYPE = "type";
   public static final String QUERY = "query";
   public static final String NAMESPACES = "namespaces";
   public static final String CONFIGURATION = "configuration";
   public static final String META = "meta";

   public static final String MISSING_COLLECTION_PARAM = "Required parameter 'collection' not found.";
   public static final String MISSING_NAME_PARAM = "Required parameter 'name' not found.";
   public static final String MISSING_DOCUMENT_PARAM = "Required parameter 'document' not found.";
   public static final String MISSING_PATTERN_PARAM = "Required parameter 'pattern' not found.";
   public static final String MISSING_TYPE_PARAM = "Required parameter 'type' not found.";
   public static final String MISSING_QUERY_PARAM = "Required parameter 'query' not found.";
   public static final String MISSING_TIMESTAMP_PARAM = "For compressed results a timestamp must be provided.";
   public static final String MISSING_CONFIGURATION_PARAM = "You must either provide a document containing the configuration or specify the 'name' and 'patter' parameters.";
   public static final String MISSING_META_CONFIGURATION = "Meta information requested but not enabled";
   public static final String MISSING_META_PARAM = "Required parameter 'meta' not found";

   /**
    * Retrieves a Collection instance based on the path provided in name.
    *
    * @param name The collection to retrieve
    * @return The Collection value
    * @exception Exception
    */
   protected Collection getCollection( String name ) throws Exception {
      // Get rid of any trailling slashes.
      while (name.endsWith("/")) {
         name = name.substring(0, name.lastIndexOf("/"));
      }
      
      // name must start with a /
      if ( name.startsWith( "/" ) ) {
         // find the database name. We just skip the first slash
         int colIndex = name.indexOf( '/', 1 );

         // We assume there's no collection specified
         String dbName =  name.substring( 1 );;
         String colName = "/";

         // if colIndex isn't -1 then we need to pick out the db and collection
         if ( colIndex != -1 ) {         
            dbName = name.substring( 1, colIndex );

            // The rest of the name locates the collection
            colName = name.substring( colIndex + 1 );
         }

         Database db = Database.getDatabase( dbName );
         if ( db == null ) {
            throw new Exception( "Database " + dbName + " could not be found" );
         }
         Collection col = db.getCollection( colName );
         if ( col == null ) {
            throw new Exception( "Collection " + colName + " could not be found" );
         }

         return col;
      }
      else {
         throw new Exception( "Collection name must begin with a '/'" );
      }
   }   
}
