package org.apache.xindice.server.rpc.messages;

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: CreateIndexer.java,v 1.4 2002/12/30 11:06:06 jbates Exp $
 */

import org.apache.xindice.core.Collection;
import org.apache.xindice.server.rpc.RPCDefaultMessage;
import org.apache.xindice.util.Configuration;
import org.apache.xindice.xml.dom.DOMParser;
import org.apache.xindice.xml.dom.DocumentImpl;

import org.w3c.dom.*;

import java.util.Hashtable;

/**
 *
 */
public class CreateIndexer extends RPCDefaultMessage {
      
   public Hashtable execute(Hashtable message) throws Exception {

      if ( ! message.containsKey(COLLECTION) ) {
         throw new Exception(MISSING_COLLECTION_PARAM);
      }

      if ( ! message.containsKey(CONFIGURATION) && ! message.containsKey(NAME) ) {
         throw new Exception(MISSING_CONFIGURATION_PARAM);
      }

      Document doc;
      // Handle the case where we're getting an entire configuration document.
      if ( message.containsKey(CONFIGURATION) ) {
         doc = DOMParser.toDocument((String) message.get(CONFIGURATION));
         if ( doc == null ) {
            throw new Exception( "Unable to parse Indexer configuration" );
         }
         
         // TODO: should extract the name from the document. done.
         String name = doc.getDocumentElement().getAttribute("name");
         message.put(NAME, name);
      }
      // Otherwise we need to build the configuration from the name and pattern.
      else {
         doc = new DocumentImpl();

         // Create the index element to hold attributes
         Element index = doc.createElement("index");

         index.setAttribute("class", "org.apache.xindice.core.indexer.ValueIndexer");
         index.setAttribute("name", (String) message.get(NAME));
         index.setAttribute("pattern", (String) message.get(PATTERN));

         doc.appendChild(index);         
      }
      
      Collection col = getCollection( (String) message.get(COLLECTION) );
      Configuration config = new Configuration(doc.getDocumentElement(), false);     
      col.createIndexer( config );
      
      Hashtable result = new Hashtable();
      result.put(RESULT, (String) message.get(NAME));
      return result;
   }
}
